local L = LibStub('AceLocale-3.0'):GetLocale('ProfessionsVault')
local addonName = L["ProfessionsVault"]
ProfessionsVault = LibStub("AceAddon-3.0"):NewAddon(addonName)
local addon = ProfessionsVault 
local AceGUI = LibStub("AceGUI-3.0")
local LDB = LibStub("LibDataBroker-1.1")
local minimapIcon = LibStub("LibDBIcon-1.0")
local addonNam
PV_svnrev = {}
PV_svnrev["ProfessionsVault.lua"] = tonumber(("$Revision: 32 $"):match("%d+"))
local DB_VERSION_MAJOR = 1
local DB_VERSION_MINOR = 1

local defaults = {
  profile = {
    debug = false, -- for addon debugging
    autoclose = false, -- close win after using a link
    tooltips = true, -- show tooltips in the ProfessionsVault window
    locked = false, -- frame lock
    pos = {
      height = 432,
      width  = 510,
      left   = 350,
      top    = 823,
    },
    treestatus = {
      treewidth = 230,
    },
    minimap = {
      hide = false,
    },
  }
}

local headerfont = GameFontHighlightLarge
local bodyfont = GameFontHighlightLarge
local profpages = false -- show a page for each profession
local activateontree = true

BINDING_NAME_PROFESSIONSVAULT = L["Show/Hide the ProfessionsVault window"]
BINDING_HEADER_PROFESSIONSVAULT = "ProfessionsVault"

local primaryProf = { 
        [2259] = 44, -- Alchemy
	[2018] = 87, -- Blacksmithing
	[7411] = 51, -- Enchanting
	[4036] = 51, -- Engineering
	[9134] = 0, -- Herbalism
	[45357] = 73, -- Inscription 
	[25229] = 84, -- Jewelcrafting 
	[2108] = 89, -- Leatherworking
	[2575] = 0, -- Mining
	[2656] = 1, -- Smelting
	[3908] = 73, -- Tailoring
}
local secondaryProf = { 
	[78670] = 1, -- Archaeology 
	[2550] = 31, -- Cooking
	[3273] = 6, -- First Aid 
	[7620] = 0, -- Fishing
}
local function translateIDs(t) 
  local r = {}
  for k,v in pairs(t) do
    local name, _, icon = GetSpellInfo(k)
    if not name then
      debug("Failed to find spellID "..k)
    else
      r[name] = {}
      r[name].icon = icon
      r[name].spellid = k
      r[name].patlen = v
    end
  end
  return r
end
primaryProf = translateIDs(primaryProf)
secondaryProf = translateIDs(secondaryProf)
local allProf = {}
local allProfSorted = {}
do 
  for p,v in pairs(primaryProf) do allProf[p]=v ; table.insert(allProfSorted, p) end
  table.sort(allProfSorted)
  local tmp = {}
  for p,v in pairs(secondaryProf) do allProf[p]=v ; table.insert(tmp, p)end
  table.sort(tmp)
  for _,v in ipairs(tmp) do table.insert(allProfSorted, v) end
  --myprint(allProfSorted)
end


local DB, DBc, settings
local charName
local optionsFrame

local function chatMsg(msg)
     DEFAULT_CHAT_FRAME:AddMessage(addonName..": "..msg)
end
local function debug(msg)
  if settings.debug then
     chatMsg(msg)
  end
end

local function myOptions() 
return {
  type = "group",
  args = {
    debug = {
      name = L["Debug"],
      desc = L["Toggle debugging output"],
      type = "toggle",
      guiHidden = true,
      set = function(info,val) settings.debug = val end,
      get = function(info) return settings.debug end
    },
    reset = {
      name = L["Reset"],
      desc = L["Reset the database"],
      type = "execute",
      order = 120,
      func = function() addon:Reset(); end,
    },
    dump = {
      name = L["Dump"],
      desc = L["Dump the database to chat window"],
      type = "execute",
      guiHidden = true,
      func = function() addon:Dump()  end,
    },
    oheader = {
      name = L["Options"],
      type = "header",
      cmdHidden = true,
      order = 0,
    },
    aheader = {
      name = L["Actions"],
      type = "header",
      cmdHidden = true,
      order = 100,
    },
    show = {
      name = L["Show"],
      desc = L["Show/Hide the ProfessionsVault window"],
      type = "execute",
      order = 110,
      func = function() addon:ToggleWindow()  end,
    },
    kheader = {
      name = L["Keybinding"],
      type = "header",
      cmdHidden = true,
      order = 200,
    },
    togglebind = {
      desc = L["Bind a key to toggle the ProfessionsVault window"],
      type = "keybinding",
      name = L["Show/Hide the ProfessionsVault window"],
      cmdHidden = true,
      order = 210,
      width = double,
      set = function(info,val) 
         local b1, b2 = GetBindingKey("PROFESSIONSVAULT")
         if b1 then SetBinding(b1) end
         if b2 then SetBinding(b2) end
         SetBinding(val, "PROFESSIONSVAULT") 
	 SaveBindings(GetCurrentBindingSet())
      end,
      get = function(info) return GetBindingKey("PROFESSIONSVAULT") end
    },
    config = {
      name = L["Config"],
      desc = L["Show the ProfessionsVault configuration window"],
      type = "execute",
      guiHidden = true,
      func = function() addon:Config()  end,
    },
    autoclose = {
      name = L["Autoclose"],
      desc = L["Automatically close the ProfessionsVault window when opening a profession"],
      type = "toggle",
      order = 10,
      set = function(info,val) settings.autoclose = val end,
      get = function(info) return settings.autoclose end
    },
    tooltips = {
      name = L["Tooltips"],
      desc = L["Show tooltips in the ProfessionsVault window"],
      type = "toggle",
      order = 20,
      set = function(info,val) settings.tooltips = val end,
      get = function(info) return settings.tooltips end
    },
    minimap = {
      name = L["Minimap icon"],
      desc = L["Show minimap icon"],
      type = "toggle",
      order = 30,
      get = function(info) return not settings.minimap.hide end,
      set = function(info,val) 
                  settings.minimap.hide = not val
		  if settings.minimap.hide then
		    minimapIcon:Hide(addonName)
		  else
		    minimapIcon:Show(addonName)
		  end
            end
    },
  }
} 
end

function addon:RefreshChar()
  DB.chars[charName] = DB.chars[charName] or {}
  DBc = DB.chars[charName]
  DBc.data = DBc.data or {}
  DBc.data.name = charName
  DBc.data.faction = UnitFactionGroup("player")
  DBc.data.level = UnitLevel("player")
  DBc.data.class = UnitClass("player")
  DBc.data.scanned = DBc.data.scanned or false
end

function addon:RefreshConfig()
  debug("RefreshConfig")
  charName = UnitName("player")
  DB = self.db.realm
  DB.chars = DB.chars or {}

  settings = addon.db.profile
  settings.vermaj = settings.vermaj or DB_VERSION_MAJOR
  settings.vermin = settings.vermin or DB_VERSION_MINOR

  addon:RefreshChar()
end

local function resetSettings() 
  debug("resetSettings")
  local ishown = InterfaceOptionsFrame:IsShown()
  local wshown = addon.gui and true
  if wshown then
    addon:ToggleWindow()
  end
  if ishown then
    InterfaceOptionsFrame:Hide();
  end
  for k,v in pairs(addon.db.profile) do
    addon.db.profile[k] = nil
  end
  for k,v in pairs(defaults.profile) do
    addon.db.profile[k] = v
  end
  addon:RefreshConfig()
  if ishown then
    InterfaceOptionsFrame_OpenToCategory(optionsFrame)
  end
  if wshown then
    addon:ToggleWindow()
  end
end

function addon:OnInitialize()
  self.db = LibStub("AceDB-3.0"):New("ProfessionsVaultDB", defaults)
  --LibStub("AceConfigRegistry-3.0"):ValidateOptionsTable(myOptions(), addonName)
  LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, myOptions(), {L["professionsvault"], L["pv"]})
  optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName)
  optionsFrame.default = resetSettings
  LibStub("AceEvent-3.0"):Embed(addon)
  settings = addon.db.profile

  debug("OnInitialize")

  self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
  self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
  self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
  self.db.RegisterCallback(self, "OnDatabaseReset", "RefreshConfig")
  self:RefreshConfig()
end

function addon:SetupVersion()
   local svnrev = 0
   PV_svnrev["X-Build"] = tonumber((GetAddOnMetadata("ProfessionsVault", "X-Build") or ""):match("%d+"))
   PV_svnrev["X-Revision"] = tonumber((GetAddOnMetadata("ProfessionsVault", "X-Revision") or ""):match("%d+"))
   for _,v in pairs(PV_svnrev) do -- determine highest file revision
     if v and v > svnrev then
       svnrev = v
     end
   end
   ProfessionsVault.revision = svnrev

   PV_svnrev["X-Curse-Packaged-Version"] = GetAddOnMetadata("ProfessionsVault", "X-Curse-Packaged-Version")
   PV_svnrev["Version"] = GetAddOnMetadata("ProfessionsVault", "Version")
   ProfessionsVault.version = PV_svnrev["X-Curse-Packaged-Version"] or PV_svnrev["Version"] or "@"
   if string.find(ProfessionsVault.version, "@") then -- dev copy uses "@.project-version.@"
      ProfessionsVault.version = "r"..svnrev
   end
end


function addon:OnEnable()
  debug("OnEnable")

  self:SetupVersion()

  self:RegisterEvent("TRADE_SKILL_SHOW")

  --if not DBc.data.scanned then -- no professions registered on this toon
  --  addon:ScanProfessions()
  --end

  local PVLDB = LDB:NewDataObject(addonName, {
        type = "launcher",
        label = addonName,
        icon = "Interface\\Icons\\spell_shadow_mindtwisting",
        OnClick = function(self, button)
                if button == "RightButton" then
			addon:Config()
                else
                        addon:ToggleWindow()
                end
        end,
        OnTooltipShow = function(tooltip)
                if tooltip and tooltip.AddLine and settings.tooltips then
                        tooltip:SetText(addonName.." "..ProfessionsVault.version)
                        tooltip:AddLine(L["|cffff8040Left Click|r to toggle the window"])
                        tooltip:AddLine(L["|cffff8040Right Click|r for config"])
                        tooltip:Show()
                end
        end,
  })

  settings.minimap = settings.minimap or {}
  minimapIcon:Register(addonName, PVLDB, settings.minimap)
  if settings.minimap.hide then
    minimapIcon:Hide(addonName)
  else
    minimapIcon:Show(addonName)
  end

end

function addon:TRADE_SKILL_SHOW()
  debug("TRADE_SKILL_SHOW")
  addon:UpdateTrade()
end

function addon:UpdateTrade()
  local name, rank, rankmax = GetTradeSkillLine()
  local link = GetTradeSkillListLink()
  local linked, linkedName = IsTradeSkillLinked()
  if (linked) then
    debug("Ignoring "..linkedName.."'s "..name);
    return
  end
  if (name and name ~= "UNKNOWN") then
    if not link then
      if name == GetSpellInfo(2575) then -- Rename mining to smelting
        name = GetSpellInfo(2656)
      end
      chatMsg(format(L["Warning: Tradeskill %s has no link available to be used."]..L["This is a known Blizzard bug."],name));
    end
    DBc[name] = {}

    DBc[name].link = link
    DBc[name].rank = rank
    DBc[name].rankmax = rankmax
    DBc[name].lastupdate = time()
    debug("Updated "..charName.."'s "..name);
  end
end

-- traversal function for character table
local cnext_sorted_names = {}
local function cnext(t,i)
   -- return them in reverse order
   if #cnext_sorted_names == 0 then 
     return nil
   else
      local n = cnext_sorted_names[#cnext_sorted_names]
      table.remove(cnext_sorted_names, #cnext_sorted_names)
      return n, t[n]
   end
end
local function cpairs(t)
  cnext_sorted_names = {}
  for n,_ in pairs(t) do
    table.insert(cnext_sorted_names, n)
  end
  table.sort(cnext_sorted_names, function (a,b) return b == charName or (a ~= charName and a > b) end)
  --myprint(cnext_sorted_names)
  return cnext, t, nil
end

function addon:Dump()
  for cname,dbc in cpairs(DB.chars) do
    local data = dbc.data or {}
    chatMsg("-=- "..cname.." "..(data.level or "").." "..(data.faction or "").." "..(data.class or "").." -=-")
    for _,pname in ipairs(allProfSorted) do
      if (dbc[pname]) then
        local pinfo = dbc[pname]
        local rank = pinfo.rank or "??"
        local rankmax = pinfo.rankmax or "??"
	local link = pinfo.link or "["..pname.."]"
        chatMsg(" "..link.."   ("..rank.."/"..rankmax..")")
      end
    end
  end
end

local function OnClose(widget)
  debug("OnClose")
  widget.frame:SetFrameStrata(widget.acegui_strata)
  addon:clearEscapeHandler(widget)
  addon:decustomizeLayout(widget)
  if settings.locked then
    addon:unlockWidget(addon.gui)
  end
  AceGUI:Release(widget)
  addon.gui = nil
end

local function cleanlink(link)
  local res = string.match(link,"|H(trade:[^|]+)|h")
  debug("cleanlink("..link..") => ".. res)
  return res
end

local function ActivateLink(cname,pname,link)
  debug("ActivateLink: "..cname.." "..pname)
  if IsShiftKeyDown() then
    local activeEditBox =  ChatEdit_GetActiveWindow();
    if activeEditBox then
      if not link then
        chatMsg(format(L["Warning: Tradeskill %s has no link available to be used."]..L["This is a known Blizzard bug."],pname));
      elseif (not ChatEdit_InsertLink(link) ) then
         debug("Failed to add tradeskill link to "..activeEditBox:GetName())
	 return
      end
    else
      return
    end
  else
    if cname == charName then
      CastSpellByName(pname)
    else
      local name, rank, rankmax = GetTradeSkillLine()
      local linked, linkedName = IsTradeSkillLinked()
      if linked and linkedName == cname and name and name == pname then -- already open so close it
        CloseTradeSkill()
      elseif not link then
        chatMsg(format(L["Warning: Tradeskill %s has no link available to be used."]..L["This is a known Blizzard bug."],pname));
      else -- open it (undocumented API - sshhh)
        SetItemRef(cleanlink(link),link,"LeftButton",ChatFrame1)
      end
    end
  end
  if settings.autoclose then
    addon.gui:Hide()
  end
end

function addon:ScanProfessions()
  debug("ScanProfessions")
  CloseTradeSkill()
  DB.chars[charName] = {}
  addon:RefreshChar()
  local gotone = false
  for p in pairs(allProf) do
    if (allProf[p].patlen > 0) then
      CastSpellByName(p) 
      CloseTradeSkill()
      if DBc[p] then
        gotone = true
      end
    end
  end
  if gotone then
    chatMsg(L["Profession Scan complete!"])
    DBc.data.scanned = true
  else
    chatMsg(L["No professions detected! (try again later?)"])
  end
  if addon.gui then
    addon:ToggleWindow()
    addon:ToggleWindow()
  end
end

local function ProfTooltip(frame, cname, pname, pinfo)
   if not frame then
     debug("Missing frame on ProfTooltip")
     return
   end
   addon.gui:SetStatusText(cname.." / "..pname)
   if not settings.tooltips then 
     return
   end
   --GameTooltip:SetOwner(frame, "ANCHOR_CURSOR"); 
   GameTooltip:SetOwner(frame, "ANCHOR_BOTTOMRIGHT"); 
   GameTooltip:SetText(pname.." ("..pinfo.rank.." / "..pinfo.rankmax..")"); 
   if pinfo.lastupdate then
     GameTooltip:AddLine("|cff00ff00"..L["Last Scan"]..": "..date(nil,pinfo.lastupdate).."|r")
   end
   GameTooltip:AddLine(L["|cffff8040Click|r to open"])
   GameTooltip:AddLine(L["|cffff8040Shift-Click|r to link in chat"])
   GameTooltip:Show() 
end

local function ProfIcon(cname,pname,pinfo)
  local icon = AceGUI:Create("Icon")
  icon:SetImage(allProf[pname].icon)
  icon.frame:RegisterForClicks("AnyDown") -- cheat to get right-clicks too
  icon:SetCallback("OnClick", function(button) ActivateLink(cname,pname,pinfo.link) end)
  icon:SetCallback("OnEnter", function(button) ProfTooltip(button.frame, cname, pname, pinfo) end)
  icon:SetCallback("OnLeave", function(button) addon.gui:SetStatusText(""); GameTooltip:Hide() end)
  return icon
end

local function MakeButton(text, fn) 
  local button = AceGUI:Create("Button")
  button:SetText(text)
  button:SetWidth(175)
  button:SetCallback("OnClick",fn)
  return button
end

local function parsePath(path)
  local cname, pname = ("\001"):split(path)
  return cname, pname
end

local function OnSelect(widget, event, value)
  local cname, pname = parsePath(value)
  if pname then
    debug("OnSelect: "..cname.." / "..pname)
    addon.gui:SetStatusText(cname.." / "..pname)
    settings.treepath = { cname , pname }
  else
    cname = value
    debug("OnSelect: "..cname)
    addon.gui:SetStatusText(cname)
    settings.treepath = { cname }
  end
  local dbc = DB.chars[cname]
  if not pname or not settings.profpages then
    widget:ReleaseChildren()
    widget:SetLayout("List") 
    -- body for a character
    local label = AceGUI:Create("Label")
    label:SetText(cname.."'s Professions\n")
    label:SetFullWidth(true)
    label:SetFontObject(headerfont)
    widget:AddChild(label)
    if cname == charName and (not dbc.data or not dbc.data.scanned) then
      widget:AddChild(MakeButton(L["Scan My Professions"], function () addon:ScanProfessions(); end))
      return
    end
    local scroller = AceGUI:Create("ScrollFrame")
    scroller:SetLayout("List")
    scroller:SetFullHeight(true)
    scroller:SetFullWidth(true)
    widget:AddChild(scroller)
    for i=1,2 do
      local label = AceGUI:Create("Label")
      label:SetText(i==1 and "Primary:\n" or "Secondary:\n")
      label:SetFontObject(bodyfont)
      scroller:AddChild(label)
      local body = AceGUI:Create("SimpleGroup")
      body:SetLayout("Flow")
      body:SetFullWidth(true)
      --body:SetAutoAdjustHeight(true)
      for pname in pairs((i==1 and primaryProf) or (i==2 and secondaryProf)) do
        if (dbc[pname]) then
	  local pinfo = dbc[pname]
          body:AddChild(ProfIcon(cname,pname,pinfo))
	end
      end
      scroller:AddChild(body)
    end
    return
  end

 
  local pinfo = DB.chars[cname][pname]

  widget:ReleaseChildren()
  widget:SetLayout("List") 
  
  -- body for a particular profession
  local header = AceGUI:Create("SimpleGroup")
  header:SetFullWidth(true)
  header:SetLayout("Flow") 
  header:AddChild(ProfIcon(cname,pname,pinfo))
  local label = AceGUI:Create("Label")
  label:SetText(cname.."'s "..pname.."\n")
  label:SetFontObject(headerfont)
  header:AddChild(label)
  widget:AddChild(header)

  local body = AceGUI:Create("SimpleGroup")
  body:SetFullWidth(true)
  local skill = AceGUI:Create("Label")
  skill:SetText("Skill level: "..pinfo.rank.." / "..pinfo.rankmax.."\n")
  skill:SetFontObject(bodyfont)
  skill:SetFullWidth(true)
  body:AddChild(skill)
  local link = AceGUI:Create("InteractiveLabel")
  link:SetCallback("OnClick", function(button) ActivateLink(cname,pname,pinfo.link) end)
  link:SetText("Skill link: "..pinfo.link)
  link:SetFontObject(bodyfont)
  link:SetFullWidth(true)
  body:AddChild(link)
  widget:AddChild(body)
end
----------------------------------------------------------------------------------
-- AceGUI hacks --

-- hack-around for the fact AceGUI:Frame is too stupid to resize its title area to fit the text
function addon:fixTitle() 
  local tit = nil
  for _,v in pairs({ addon.gui.frame:GetRegions() }) do 
     local w = v:GetWidth()
     if w > 90 and w < 110 then 
        v:SetWidth(200) 
	tit = v
	break;
     end 
  end
 if false then 
  -- adjust the drag handle - this part should work but doesnt
  for _,v in pairs({ addon.gui.frame:GetChildren() }) do 
     local  w = v:GetWidth()
     local  h = v:GetHeight()
     if w > 95 and w < 105 and h > 35 and h < 45 then 
        v:SetAllPoints(tit) 
        v:SetWidth(400) 
     end 
  end
 end
end 

-- hack-around for the fact AceGUI doesnt allow frame locking
function addon:lockWidget(widget) 
  widget.lockinfo = {}
  for _,v in pairs({ widget.frame:GetChildren() }) do 
     local md = v:GetScript("OnMouseDown")
     local mu = v:GetScript("OnMouseUp")
     if md and mu then
       table.insert(widget.lockinfo,{frame=v,OnMouseDown=md,OnMouseUp=mu})
       v:SetScript("OnMouseDown",function()end)
       v:SetScript("OnMouseUp",function()end)
     end
  end
end
function addon:unlockWidget(widget) 
  if widget.lockinfo then
    for _,li in pairs(widget.lockinfo) do 
      li.frame:SetScript("OnMouseDown",li.OnMouseDown)
      li.frame:SetScript("OnMouseUp",li.OnMouseUp)
    end
    widget.lockinfo = nil
  end
end

-- hack to hook the escape key for closing the window
function addon:setEscapeHandler(widget, fn)
  widget.origOnKeyDown = widget.frame:GetScript("OnKeyDown")
  widget.frame:SetScript("OnKeyDown", function(self,key) 
        if key == "ESCAPE" then 
	   fn()
	elseif widget.origOnKeyDown then
	   widget.origOnKeyDown(self,key)
	end 
     end)
  widget.frame:EnableKeyboard(true)
  widget.frame:SetPropagateKeyboardInput(true)
end
function addon:clearEscapeHandler(widget)
  widget.frame:SetScript("OnKeyDown", widget.origOnKeyDown)
  widget.frame:EnableKeyboard(false)
  widget.frame:SetPropagateKeyboardInput(false)
end
-- allow custom layout of children
function addon:customizeLayout(widget, fn)
  widget.origLayoutFinished = widget.LayoutFinished
  widget.LayoutFinished = function () if widget.origLayoutFinished then widget.origLayoutFinished() end ; fn(); end
end
function addon:decustomizeLayout(widget)
  if widget.LayoutFinished then
    widget.LayoutFinished = widget.origLayoutFinished
    widget.origLayoutFinished = nil
  end
end
----------------------------------------------------------------------------------
function addon:MakeHeaderButton(img, imgH, clickfn)
  local button = AceGUI:Create("InteractiveLabel")

  button:SetImage(img)
  button:SetHighlight(imgH)
  button:SetImageSize(32,32)
  button:SetWidth(32)
  button:SetHeight(32)

  button:SetCallback("OnClick", clickfn)

  return button
end

local icon_close = "Interface\\Buttons\\UI-Panel-MinimizeButton-Up"
local icon_closeH = "Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight"
local icon_unlocked  = "Interface\\AddOns\\ProfessionsVault\\images\\LockButton-Unlocked"
local icon_locked  =   "Interface\\AddOns\\ProfessionsVault\\images\\LockButton-Locked"
function addon:ToggleLock(button)
  if settings.locked then
    debug("frame unlocked");
    settings.locked = false
    addon:unlockWidget(addon.gui)
    button:SetImage(icon_unlocked)
    button:SetHighlight(icon_closeH)
  else
    debug("frame locked");
    settings.locked = true
    addon:lockWidget(addon.gui)
    button:SetImage(icon_locked)
    button:SetHighlight(icon_closeH)
  end
end

function addon:CreateWindow()
  if addon.gui then
    return
  end
  local f = AceGUI:Create("Frame")
  addon.gui = f
  f:SetTitle(L["ProfessionsVault"].." "..ProfessionsVault.version)
  f:SetStatusText(L["ProfessionsVault listing of your toon professions"])
  addon:fixTitle()
  f:SetCallback("OnClose", OnClose)
  f:SetLayout("Flow")
  f.frame:SetClampedToScreen(true)
  settings.pos = settings.pos or {}
  f:SetStatusTable(settings.pos)
  f:SetAutoAdjustHeight(true)
  f.acegui_strata = f.frame:GetFrameStrata()
  f.frame:SetFrameStrata("MEDIUM")

  --f:AddChild(MakeButton(L["Profession Pane"], function () ToggleSpellBook(BOOKTYPE_PROFESSION); end)) 
  f:AddChild(MakeButton(L["Scan My Professions"], function () addon:ScanProfessions(); end))
  f:AddChild(MakeButton(L["Config"], function () addon:Config(); end))

  local closeButton = addon:MakeHeaderButton(icon_close, icon_closeH,
                          function() addon:ToggleWindow() end)
  f:AddChild(closeButton)
  local lockButton = addon:MakeHeaderButton(icon_unlocked, icon_closeH,
                          function(self) addon:ToggleLock(self) end)
  f:AddChild(lockButton)
  addon:customizeLayout(f, function() 
     closeButton.frame:ClearAllPoints(); 
     closeButton:SetPoint("TOPRIGHT",f.frame,-5,-5)
     lockButton.frame:ClearAllPoints(); 
     lockButton:SetPoint("TOPRIGHT",closeButton.frame,"TOPLEFT",8,0)
  end)

  if settings.locked then
    settings.locked = false
    addon:ToggleLock(lockButton)
  end

  addon:setEscapeHandler(f, function() addon:ToggleWindow() end)

  local treedata = {}
  for cname,dbc in cpairs(DB.chars) do
    local ct = { value = cname, text = cname, 
                 icon = "Interface\\GossipFrame\\ActiveQuestIcon",
                 children = {} }
    --print(cname)
    if dbc.data and dbc.data.faction and dbc.data.faction == "Horde" then
      ct.icon = "Interface\\TargetingFrame\\UI-PVP-Horde"
    elseif dbc.data and dbc.data.faction and dbc.data.faction == "Alliance" then
      ct.icon = "Interface\\TargetingFrame\\UI-PVP-Alliance"
    end
    for _,pname in ipairs(allProfSorted) do
      if (dbc[pname]) then
        local pinfo = dbc[pname]
        table.insert(ct.children, { value = pname, icon = allProf[pname].icon,
                                      text = pname.." ("..pinfo.rank.." / "..pinfo.rankmax..")" })
      end
    end
    table.insert(treedata, ct)
  end
  if settings.debug then
    myprint(treedata)
  end

  local tree = AceGUI:Create("TreeGroup")
  tree:SetFullWidth(true)
  tree:SetFullHeight(true)
  --tree:SetAutoAdjustHeight(true)
  tree:SetTree(treedata)
  tree:SetCallback("OnGroupSelected", OnSelect)
  tree:SetCallback("OnClick", function (widget, event, path)
  	local cname, pname = parsePath(path)
  	if pname and activateontree then
  	  ActivateLink(cname,pname,DB.chars[cname][pname].link)
  	end  
  end)
  tree:SetCallback("OnButtonEnter", function(button,event,path,frame) 
                   local cname, pname = parsePath(path)
		   addon.gui:SetStatusText(cname)
		   if pname then
                     ProfTooltip(frame, cname, pname, DB.chars[cname][pname]) 
		   elseif cname and settings.tooltips then
		     local data = DB.chars[cname].data
                     GameTooltip:SetOwner(frame, "ANCHOR_BOTTOMRIGHT");
                     GameTooltip:SetText(cname);
                     local text = ""
                     for _,stat in ipairs({"level","faction","class"}) do
                       if data and data[stat] then
                          text = text..data[stat].." "
                       end
                     end
                     if #text then
                        GameTooltip:AddLine(text)
                     end
                     GameTooltip:Show()
		   end
		   end)
  tree:SetCallback("OnButtonLeave", function() GameTooltip:Hide() end)
  tree:EnableButtonTooltips(false)
  f:AddChild(tree)
  settings.treestatus = settings.treestatus or {}
  tree:SetStatusTable(settings.treestatus)
  if settings.treepath then
    local oldact = activateontree
    activateontree = false
    tree:SelectByPath(unpack(settings.treepath))
    activateontree = oldact
  end
end

function addon:ToggleWindow()
  debug("ToggleWindow")

  if addon.gui then
    addon.gui:Hide()
  else
    addon:CreateWindow()
  end
end

function addon:Reset() 
  if addon.gui then
    addon:ToggleWindow()
  end
  StaticPopup_Show("PROFESSIONSVAULT_RESET")
end

local function ResetConfirmed()
  addon.db:ResetDB() -- triggers RefreshConfig
  chatMsg(L["Reset complete."]); 
end

StaticPopupDialogs["PROFESSIONSVAULT_RESET"] = {
  text = L["Are you sure you want to reset the ProfessionsVault database and wipe all data?"],
  button1 = OKAY,
  button2 = CANCEL,
  OnAccept = ResetConfirmed,
  timeout = 0,
  whileDead = true,
  hideOnEscape = true,
}

function addon:Config() 
  if optionsFrame then
    if ( InterfaceOptionsFrame:IsShown() ) then
      InterfaceOptionsFrame:Hide();
    else
      InterfaceOptionsFrame_OpenToCategory(optionsFrame)
    end
  end
end
----------------------------------------------------------------------------
-- Testing code
----------------------------------------------------------------------------
function addon:fakeLink(profName, spellid, rank, rankmax, guid) 
  spellid = spellid or allProf[profName].spellid
  CastSpellByName("Cooking")
  CloseTradeSkill()
  for i=0,200 do
    local known = string.rep("/",i)
    local link = "\124Htrade:"..spellid..":"..rank..":"..rankmax..":"..
          guid..":"..known.."\124h["..profName.."]\124h"
    --print(i..": "..link)
    SetItemRef(cleanlink(link),link,"LeftButton",ChatFrame1)
    if GetTradeSkillLine() and IsTradeSkillLinked() then
      print("Success at "..spellid.." "..i.." "..GetSpellInfo(spellid).." "..link)
      --printlink(link)
      break
    end
  end
end

function PV_link(spellname,spellid) 
  spellname = spellname or "Smelting"
  addon:fakeLink(spellname,spellid,525,525,"680000000256861")
end

function PV_linkscan(min,max) 
  local known = {}
  for k,v in pairs(allProf) do
      known[v.spellid] = k
  end
  for i=min,max do
    if known[i] and false then
      print("Skipping "..i.." "..known[i])
    else
      if i%100 == 0 then
        print("testing "..i)
      end
      go("linkscan",i)
    end
  end
  print("Scan complete: "..min..".."..max)
end
----------------------------------------------------------------------------


